from datetime import datetime
from uuid import UUID, uuid4

import pytest
from psycopg2 import IntegrityError

from hou_flask_psycopg2 import SQLNotFoundException
from oss_auth.daos.authentication_dao import (
    create_account,
    create_token,
    delete_token,
    get_account_for_token,
)
from oss_auth.database import DB
from tests.utils.factories.account import AccountFactory, RoleFactory, TokenFactory
from tests.utils.factories.base_factory import random_string


class CreateUserTest:
    def test_when_role_name_not_exists__raises_exception(self):
        with pytest.raises(SQLNotFoundException):
            create_account("not_real", random_string(), uuid4())

    def test_when_role_name_exists__returns_new_id(self):
        role = RoleFactory.build()
        account_id = create_account(role.name, random_string(), uuid4())
        assert isinstance(account_id, UUID)
        resp = DB.execute_query(
            "SELECT COUNT(id) FROM oss_auth.account WHERE id=%s",
            (account_id,),
            expect_one=True,
        )
        assert resp[0] == 1


class CreateTokenTest:
    def test_when_auth_not_exists__raises_exception(self):
        with pytest.raises(IntegrityError):
            create_token(uuid4(), random_string(), random_string(), datetime.now())

    def test_when_auth_exists__returns_token(self):
        account = AccountFactory()
        resp = create_token(
            account.id, random_string(), random_string(), datetime.now()
        )
        assert isinstance(resp, str)


class GetUserForTokenTest:
    def test_when_token_not_exists__raises_sql_not_found_exception(self):
        with pytest.raises(SQLNotFoundException):
            get_account_for_token("fake")

    def test_when_token_exists__returns_user_id(self):
        token = TokenFactory()
        resp = get_account_for_token(token.token)
        assert resp == token.account_id


class DeleteTokenTest:
    def test_when_no_token__no_error(self):
        delete_token("not-real")

    def test_when_token__deletes_token(self):
        token = TokenFactory()
        delete_token(token.token)
        resp = DB.execute_query(
            "SELECT COUNT(*) FROM oss_auth.token WHERE token.token = %s", (token.token,)
        )
        assert resp[0][0] == 0



import rapidjson


def monkeypatch_rapidjson():
    """
    Force rapidjson to always convert UUID and datetimes
    """
    _monkeypatch_dumps()
    _monkeypatch_loads()


def _monkeypatch_dumps():
    original_dumps = rapidjson.dumps

    def _dumps(
        *args, separators=None, cls=None, **kwargs
    ):  # pylint: disable=unused-argument
        return original_dumps(
            *args,
            uuid_mode=rapidjson.UM_CANONICAL,
            datetime_mode=rapidjson.DM_ISO8601,
            **kwargs
        )

    rapidjson.dumps = _dumps


def _monkeypatch_loads():
    original_loads = rapidjson.loads

    def _loads(
        *args, separators=None, cls=None, **kwargs
    ):  # pylint: disable=unused-argument
        return original_loads(
            *args,
            uuid_mode=rapidjson.UM_CANONICAL,
            datetime_mode=rapidjson.DM_ISO8601,
            **kwargs
        )

    rapidjson.loads = _loads



from unittest import mock

from oss_auth.views.meta import _check_database


class CheckDatabaseTest:
    @mock.patch("oss_auth.views.meta.DB")
    def test_when_exception__returns_error(self, database):
        database.cursor.side_effect = Exception
        resp = _check_database()
        assert not resp["available"]



import pytest

from oss_auth.daos.permission_dao import Permission
from oss_auth.exceptions import MissingPermissionsException
from oss_auth.utils.authorization import _require_permissions


class RequirePermissionsTest:
    def test_when_all_permissions__no_exception(self):
        _require_permissions(
            {Permission.get_all_orgs}, {Permission.get_all_orgs, Permission.update_org}
        )

    def test_when_missing_permissions__raises_exception(self):
        with pytest.raises(MissingPermissionsException):
            _require_permissions(
                {Permission.get_all_orgs},
                {Permission.update_org, Permission.create_org},
            )



from dataclasses import dataclass
from typing import Dict, Iterable, List, Optional, Union
from uuid import UUID

from hou_flask_psycopg2 import CustomJSON, Psycopg2Utils
from psycopg2.errors import UniqueViolation

from ...exceptions import DuplicateFeatureFlagException

_GET_ORG_VARIANT_NAME = "get_org_variant"


@dataclass
class FeatureFlag:
    name: str
    default_value: Union[int, str, bool, List, Dict]


@dataclass
class NewFeatureVariant:
    flag_name: str
    variant: Union[str, int, bool, List, Dict]


@dataclass
class FeatureVariant(NewFeatureVariant):
    id: UUID


@dataclass
class FeatureVariantOrganization:
    organization_id: UUID
    variant_id: Optional[UUID]
    flag_name: str


class FeatureFlagDAO:
    db: Psycopg2Utils

    def __init__(self, db: Psycopg2Utils):
        self.db = db

    def create_feature_flag(self, name, default_value) -> FeatureFlag:
        query = """
        INSERT INTO feature_flag (name, default_value)
        VALUES (%s, %s)
        RETURNING name, default_value
        """
        try:
            return self.db.execute_query_conversion(
                query, (name, CustomJSON(default_value)), FeatureFlag, expect_one=True
            )
        except UniqueViolation:
            raise DuplicateFeatureFlagException(name)

    def create_feature_variants(
        self, variants: Iterable[NewFeatureVariant]
    ) -> List[FeatureVariant]:
        query = """
        INSERT INTO feature_variant(flag_name, variant)
        VALUES {template}
        RETURNING id, flag_name, variant
        """
        to_insert = [(var.flag_name, CustomJSON(var.variant)) for var in variants]
        return self.db.bulk_insert(to_insert, FeatureVariant, query)

    def create_feature_variant_organization(
        self, flag_name: str, variant_id: UUID, organization_ids: Iterable[UUID]
    ) -> List[FeatureVariantOrganization]:
        query = """
        INSERT INTO feature_variant_organization(flag_name, variant_id, organization_id)
        VALUES {template}
        RETURNING flag_name, variant_id, organization_id
        """
        to_insert = [(flag_name, variant_id, org_id) for org_id in organization_ids]
        return self.db.bulk_insert(to_insert, FeatureVariantOrganization, query)

    def delete_feature_variant_organization(
        self, organization_id: UUID, flag_name: str
    ):
        query = """
        DELETE FROM feature_variant_organization
        WHERE organization_id = %s AND flag_name = %s
        """
        self.db.execute_query_without_fetch(query, (organization_id, flag_name))

    def delete_feature_variants(self, variant_ids: Iterable[UUID]):
        variant_ids = tuple(variant_ids)
        if not variant_ids:
            return
        query = "DELETE FROM feature_variant WHERE id IN %s"
        self.db.execute_query_without_fetch(query, (variant_ids,))

    def get_feature_flags(self) -> List[FeatureFlag]:
        query = """
        SELECT name, default_value
        FROM feature_flag
        """
        return self.db.execute_query_conversion(query, tuple(), FeatureFlag)

    def get_feature_variants(self) -> List[FeatureVariant]:
        query = """
        SELECT flag_name, variant, id
        FROM feature_variant
        """
        return self.db.execute_query_conversion(query, tuple(), FeatureVariant)

    def get_org_variant(self, name, organization_id):
        query = """
        SELECT COALESCE(fv.variant, ff.default_value)
        FROM feature_flag AS ff
        LEFT JOIN feature_variant_organization AS fvo 
          ON ff.name = fvo.flag_name AND organization_id = %s
        LEFT JOIN feature_variant AS fv ON ff.name = fv.flag_name
        WHERE ff.name = %s
        """
        return self.db.execute_query(query, (organization_id, name), expect_one=True)[0]



import os
from typing import Dict

from flask import Response, request
from flask_wtf.csrf import generate_csrf
from hou_flask_psycopg2 import Psycopg2Utils
from ultra_config import GlobalConfig

from ..services.authentication_service import AuthenticationService
from ..services.authorization_service import AUTH_COOKIE
from ...connexion import FlaskConnexionExtension
from ...http import json_response

__all__ = ["AUTHENTICATION_VIEW"]


class _AuthenticationView:
    authentication_service: AuthenticationService

    def init_app(self, db: Psycopg2Utils, connexion_app: FlaskConnexionExtension):
        self.authentication_service = AuthenticationService(db)
        swagger_file = os.path.join(os.path.dirname(__file__), "..", "swagger-spec.yml")
        connexion_app.add_api(swagger_file, base_path="/v1/authentication")


AUTHENTICATION_VIEW = _AuthenticationView()


def register(body: Dict) -> Response:
    password = body["password"]
    confirm_password = body["confirm_password"]
    if not password == confirm_password:
        return json_response(
            {"message": "password and confirm password must match"}, 400
        )
    AUTHENTICATION_VIEW.authentication_service.register_user(
        body["email"], body["password"]
    )
    return json_response(
        {
            "message": (
                f"An email was sent to {body['email']} "
                f"Please click the link in the email to confirm your account"
            )
        },
        201,
    )


def login(body: Dict) -> Response:
    token = AUTHENTICATION_VIEW.authentication_service.login_user(
        body["username"], body["password"]
    )
    expires_in = 60 * 60 * 24 * 30 if body["remember"] else None
    return _set_token_cookie(token, json_response({"success": True}, 201), expires_in)


def logout() -> Response:
    token = request.cookies.get(AUTH_COOKIE)
    resp = json_response({"success": True, "login_url": "/login"})
    if token:
        AUTHENTICATION_VIEW.authentication_service.logout_account(token)
        _set_token_cookie("", resp, 0)
    return resp


def get_csrf_token():
    return json_response({"csrf_token": generate_csrf()})


def _set_token_cookie(
    token: str, response: Response, expires_in: int = None
) -> Response:
    secure = GlobalConfig.config["SECURE_COOKIES"]
    response.set_cookie(
        AUTH_COOKIE, token, path="/", max_age=expires_in, secure=secure, httponly=True
    )
    return response



import dataclasses
from enum import Enum
from typing import Dict
from uuid import UUID

from flask import Response, request
from hou_flask_psycopg2 import Psycopg2Utils

from houflask.auth import AuthorizationService
from houflask.connexion import FlaskConnexionExtension
from houflask.http import (
    json_response,
    dataclass_json_response,
    empty_response,
    uuid_converter,
)
from houflask.organization.daos.organization_account_dao import (
    OrganizationAccountDAO,
    UpsertAccountOrganization,
)

from houflask.organization.daos.organization_dao import OrganizationDAO
from houflask.organization.services.organization_account_service import (
    OrganizationAccountService,
)
from ..services.organization_service import OrganizationService
import os


_AUTH = AuthorizationService()


class Permission(Enum):
    create_org = "organization:create"
    update_org = "organization:update"
    update_org_members = "organization:update:members"
    get_all_orgs = "organization:retrieve-all"
    get_org = "organization:retrieve"


class _OrganizationView:
    organization_service: OrganizationService
    organization_account_service: OrganizationAccountService

    def init_app(self, db: Psycopg2Utils, connexion_app: FlaskConnexionExtension):
        self.organization_service = OrganizationService(
            OrganizationDAO(db), OrganizationAccountDAO(db)
        )
        self.organization_account_service = OrganizationAccountService(
            OrganizationAccountDAO(db)
        )
        _AUTH.init_db(db)
        swagger_file = os.path.join(os.path.dirname(__file__), "..", "swagger-spec.yml")
        connexion_app.add_api(swagger_file, base_path="/v1/organization")


ORGANIZATION_VIEW = _OrganizationView()


@_AUTH.require_global_permissions(Permission.create_org)
def create():
    data = request.get_json()
    org = ORGANIZATION_VIEW.organization_service.create(
        data["name"], _AUTH.get_current_account(), data["active"]
    )
    return json_response(dataclasses.asdict(org), status=201)


@_AUTH.require_global_permissions(Permission.get_all_orgs)
def get_all(page, page_size, name_filter=""):
    orgs = ORGANIZATION_VIEW.organization_service.get_all_orgs(
        page, page_size, name_filter
    )
    models = [dataclasses.asdict(org) for org in orgs]
    return json_response(models, 200)


@_AUTH.require_org_permissions(Permission.get_org)
def get_org(organization_id: UUID) -> Response:
    org = ORGANIZATION_VIEW.organization_service.get_org(organization_id)
    return json_response(dataclasses.asdict(org), 200)


@_AUTH.require_global_permissions(Permission.update_org)
def update(organization_id, body):
    org = ORGANIZATION_VIEW.organization_service.update(
        organization_id, name=body["name"], active=body["active"]
    )
    return json_response(dataclasses.asdict(org))


@_AUTH.require_global_permissions(Permission.get_all_orgs)
def organization_name_available():
    available = ORGANIZATION_VIEW.organization_service.organization_name_available(
        request.args["name"]
    )
    return json_response({"available": available})


def get_accounts_for_org(
    organization_id: UUID, page: int, page_size: int, name_filter: str = ""
) -> Response:
    accounts = ORGANIZATION_VIEW.organization_account_service.get_accounts_for_org(
        organization_id, name_filter, page, page_size
    )
    return dataclass_json_response(accounts)


def get_organization_accounts_for_own_account() -> Response:
    account_id = _AUTH.get_current_account()
    return dataclass_json_response(
        ORGANIZATION_VIEW.organization_account_service.get_orgs_for_account(account_id)
    )


def get_organizations_for_own_account() -> Response:
    account_id = _AUTH.get_current_account()
    return dataclass_json_response(
        ORGANIZATION_VIEW.organization_service.get_orgs_for_account(account_id)
    )


@uuid_converter(("organization_id", "account_id"))
@_AUTH.require_org_permissions(Permission.update_org_members)
def upsert_account_org(organization_id: UUID, account_id: UUID, body: Dict) -> Response:
    upsert_row = UpsertAccountOrganization(
        organization_id, account_id, body["role_name"]
    )
    return dataclass_json_response(
        ORGANIZATION_VIEW.organization_account_service.upsert_account_orgs(
            [upsert_row]
        )[0]
    )


@_AUTH.require_org_permissions(Permission.update_org_members)
def remove_account_org(organization_id: UUID, account_id: UUID) -> Response:
    ORGANIZATION_VIEW.organization_account_service.remove_account_org(
        organization_id, account_id
    )
    return empty_response()



import base64
import hashlib
import hmac
import logging
from datetime import datetime, timedelta
from typing import Dict, Tuple
from uuid import UUID

import boto3
import rapidjson
from beaker.cache import cache_region
from hou_flask_psycopg2 import SQLNotFoundException, Psycopg2Utils
from jose import jwt
from ultra_config import GlobalConfig

from ..daos.authentication_dao import AuthenticationDAO
from ..daos.permission_dao import Role
from ...exceptions import NotLoggedInException
from ...cache import LOCAL_LONG_TERM

LOG = logging.getLogger(__name__)

__all__ = ["AuthenticationService"]


class AuthenticationService:
    db: Psycopg2Utils
    authentication_dao: AuthenticationDAO

    def __init__(self, db: Psycopg2Utils):
        self.authentication_dao = AuthenticationDAO(db)

    @GlobalConfig.inject(client_id="COGNITO_CLIENT_ID", pool_id="COGNITO_USER_POOL_ID")
    def login_user(self, username, password, client_id=None, pool_id=None) -> str:
        secret_hash = AuthenticationService._build_secret_hash(username)
        client = AuthenticationService._cognito_client()
        resp = client.admin_initiate_auth(
            AuthFlow="ADMIN_NO_SRP_AUTH",
            AuthParameters={
                "USERNAME": username,
                "PASSWORD": password,
                "SECRET_HASH": secret_hash,
            },
            ClientId=client_id,
            UserPoolId=pool_id,
        )

        id_token = resp["AuthenticationResult"]["IdToken"]
        claims = AuthenticationService._parse_jwt_token(id_token)
        return AuthenticationService._save_login(
            claims["sub"],
            claims["email"],
            resp["AuthenticationResult"]["AccessToken"],
            resp["AuthenticationResult"]["RefreshToken"],
        )

    @GlobalConfig.inject(client_id="COGNITO_CLIENT_ID")
    def register_user(self, email, password, client_id=None) -> Dict:
        secret_hash = self._build_secret_hash(email)
        client = self._cognito_client()
        resp = client.sign_up(
            ClientId=client_id,
            SecretHash=secret_hash,
            Username=email,
            Password=password,
            UserAttributes=[{"Name": "email", "Value": email}],
        )
        account, _ = self._get_or_create_account(resp["UserSub"], email)
        return account

    def _save_login(
        self, cognito_id: UUID, email: str, access_token: str, refresh_token: str
    ) -> str:
        account, _ = self._get_or_create_account(cognito_id, email)
        return self._create_token(account["id"], access_token, refresh_token)

    @GlobalConfig.inject(client_id="COGNITO_CLIENT_ID")
    def _parse_jwt_token(self, token, client_id: str = None):
        header = jwt.get_unverified_header(token)
        kid = header["kid"]
        key = AuthenticationService._jwks_json()[kid]
        return jwt.decode(token, key, audience=client_id)

    @GlobalConfig.inject(
        client_id="COGNITO_CLIENT_ID", client_secret="COGNITO_CLIENT_SECRET"
    )
    def _build_secret_hash(
        self, username: str, client_id: str = "", client_secret: str = ""
    ) -> str:
        to_digest = f"{username}{client_id}"
        digest = hmac.new(
            client_secret.encode("utf-8"),
            msg=to_digest.encode("utf-8"),
            digestmod=hashlib.sha256,
        ).digest()
        return base64.b64encode(digest).decode()

    @cache_region(LOCAL_LONG_TERM, "_cognito_client")
    @GlobalConfig.inject(region="AWS_REGION")
    def _cognito_client(self, region: str = None):
        return boto3.client("cognito-idp", region_name=region)

    @cache_region(LOCAL_LONG_TERM, "_cognito_jwks")
    @GlobalConfig.inject(jwks_file="COGNITO_JWKS_FILE")
    def _jwks_json(self, jwks_file: str = ""):
        with open(jwks_file) as jwks_json:
            json_data = rapidjson.load(jwks_json)
        return {data["kid"]: data for data in json_data["keys"]}

    def _get_or_create_account(self, cognito_id: UUID, email: str) -> Tuple[Dict, bool]:
        try:
            return AuthenticationDAO.get_account_for_cognito_id(cognito_id), False
        except SQLNotFoundException:
            LOG.info(
                'Creating a new account because a row was not found for "%s"',
                cognito_id,
            )
            self.authentication_dao.create_account(
                Role.default.value, email, cognito_id
            )
            return AuthenticationDAO.get_account_for_cognito_id(cognito_id), True

    def _create_token(
        self,
        account_id: UUID,
        access_token: str,
        refresh_token: str,
        expiration: float = None,
    ) -> str:
        if expiration is None:
            expiration = GlobalConfig.config["DEFAULT_TOKEN_EXPIRATION_SECONDS"]
        expiration_time = datetime.utcnow() + timedelta(
            seconds=expiration
        )  # type: ignore
        return self.authentication_dao.create_token(
            account_id, access_token, refresh_token, expiration_time
        )

    def logout_account(self, token: str):
        self.authentication_dao.delete_token(token)

    def get_account_by_token(self, token: str) -> UUID:
        try:
            return self.authentication_dao.get_account_for_token(token)
        except SQLNotFoundException as exc:
            raise NotLoggedInException from exc



from uuid import UUID
from typing import AbstractSet

from hou_flask_psycopg2 import SQLNotFoundException
from werkzeug.exceptions import BadRequest, NotFound, Unauthorized, Forbidden


class InvalidUsernamePasswordException(Unauthorized):
    pass


class NotLoggedInException(Unauthorized):
    pass


class ParameterNotPresentException(Exception):
    def __init__(self, function_name: str, parameter_name, *args):
        super().__init__(
            f'The "{parameter_name}" parameter was not present in the call to "{function_name}',
            *args,
        )


class OrganizationNotFound(NotFound, SQLNotFoundException):
    def __init__(self, id_: UUID, *args, **kwargs) -> None:
        super().__init__(f'Organization with id "{id_}" not found', *args, **kwargs)


class OrganizationNameUnavailable(BadRequest, SQLNotFoundException):
    def __init__(self, name: str, *args, **kwargs) -> None:
        super().__init__(f'Organization with name "{name}" not found', *args, **kwargs)


class DuplicateFeatureFlagException(BadRequest):
    def __init__(self, name):
        super().__init__(f"A feature flag with the name {name} already exists")


class MissingPermissionsException(Forbidden):
    def __init__(self, missing_permissions: AbstractSet[str], *args, **kwargs) -> None:
        missing = ", ".join(missing_permissions)
        super().__init__(
            f"The following permissions are missing: {missing}", *args, **kwargs
        )



from typing import List
from uuid import UUID

from houflask.auth.daos.permission_dao import Role
from ..daos.organization_account_dao import (
    OrganizationAccountDAO,
    UpsertAccountOrganization,
)

from ..daos.organization_dao import Organization, OrganizationDAO


class OrganizationService:
    organization_dao: OrganizationDAO
    organization_account_dao: OrganizationAccountDAO

    def __init__(
        self,
        organization_dao: OrganizationDAO,
        organization_account_dao: OrganizationAccountDAO,
    ):
        self.organization_dao = organization_dao
        self.organization_account_dao = organization_account_dao

    def create(self, name: str, owner_id: UUID, active: bool = True) -> Organization:
        org = self.organization_dao.create(name, active)
        self.organization_account_dao.upsert_account_orgs(
            [UpsertAccountOrganization(org.id, owner_id, Role.superuser.value)]
        )
        return org

    def update(self, id_: UUID, name: str, active: bool) -> Organization:
        return self.organization_dao.update(id_, name, active)

    def get_all_orgs(
        self, page: int, page_size: int, name_filter: str = ""
    ) -> List[Organization]:
        return self.organization_dao.get_all_orgs(page, page_size, name_filter)

    def get_orgs_for_account(self, account_id: UUID):
        return self.organization_dao.get_orgs_for_account(account_id)

    def get_org(self, organization_id: UUID) -> Organization:
        return self.organization_dao.get_org(organization_id)

    def organization_name_available(self, name: str) -> bool:
        return not self.organization_dao.org_name_exists(name)



import secrets
from dataclasses import dataclass
from datetime import datetime
from typing import Dict
from uuid import UUID

from hou_flask_psycopg2 import SQLNotFoundException, Psycopg2Utils
from psycopg2 import IntegrityError
from psycopg2.extras import RealDictCursor

__all__ = ["AuthenticationDAO"]


@dataclass
class Account:
    username: str
    id: UUID


@dataclass(order=True, frozen=True, unsafe_hash=True)
class Username:
    account_id: UUID
    username: str


class AuthenticationDAO:
    db: Psycopg2Utils

    def __init__(self, db: Psycopg2Utils):
        self.db = db

    def get_account_for_cognito_id(self, cognito_id: UUID) -> Dict:
        return self.db.execute_query(
            """
            SELECT id, cognito_id
            FROM account
            WHERE cognito_id = %s
            """,
            (cognito_id,),
            factory=RealDictCursor,
            expect_one=True,
        )

    def create_account(self, role_name: str, email: str, cognito_id: UUID) -> UUID:
        try:
            return self.db.execute_query(
                """
                INSERT INTO account (role_name, email, cognito_id)
                VALUES (%s, %s, %s)
                RETURNING id
                """,
                (role_name, email, cognito_id),
                expect_one=True,
            )[0]
        except IntegrityError as exc:
            raise SQLNotFoundException(f"Missing role: {role_name}") from exc

    def create_token(
        self,
        account_id: UUID,
        access_token: str,
        refresh_token: str,
        expiration: datetime,
    ) -> str:
        token = secrets.token_urlsafe(64)
        query = """
        INSERT INTO token (
          account_id, 
          token,
          expiry,
          cognito_access_token,
          cognito_refresh_token
        )
        VALUES (%s, %s, %s, %s, %s)
        """
        self.db.execute_query_without_fetch(
            query, (account_id, token, expiration, access_token, refresh_token)
        )
        return token

    def get_account_for_token(self, token: str) -> UUID:
        account_ids = self.db.execute_query(
            """
            SELECT account_id
            FROM token
            WHERE token = %s
            """,
            (token,),
        )
        if not account_ids:
            raise SQLNotFoundException("User not logged in")
        return account_ids[0][0]

    def delete_token(self, token: str) -> None:
        query = """
        DELETE FROM token
        WHERE token.token = %s
        """
        self.db.execute_query_without_fetch(query, (token,))



from collections import namedtuple
from dataclasses import dataclass
from typing import List
from uuid import UUID

from hou_flask_psycopg2 import Psycopg2Utils

from houflask.auth.daos.authentication_dao import Username

OrganizationAccountId = namedtuple(
    "OrganizationUserId", ["organization_id", "account_id"]
)


@dataclass
class Account:
    username: str
    id: UUID


@dataclass(order=True)
class OrganizationAccount:
    username: str
    organization_id: UUID
    account_id: UUID
    role_name: str


@dataclass(order=True)
class DepartmentAccount:
    username: str
    department_id: UUID
    account_id: UUID
    role_name: str


@dataclass(order=True)
class UpsertAccountOrganization:
    organization_id: UUID
    account_id: UUID
    role_name: str


class OrganizationAccountDAO:
    def __init__(self, db: Psycopg2Utils):
        self.db = db

    def get_accounts_for_org(
        self,
        organization_id: UUID,
        name_filter: str = "",
        page: int = 0,
        page_size: int = 100,
    ) -> List[OrganizationAccount]:
        query = """
        SELECT 
          account.email AS username, 
          organization_account.account_id, 
          organization_account.organization_id,
          organization_account.role_name
        FROM account
        INNER JOIN organization_account ON account.id = organization_account.account_id
        WHERE organization_id = %s
          AND account.email ILIKE %s ESCAPE '='
        ORDER BY LOWER(account.email)
        LIMIT %s
        OFFSET %s
        """
        name_filter = self.db.unsafe_escape_like_string(
            name_filter, escape_character="="
        )
        return self.db.execute_query_conversion(
            query,
            (organization_id, f"%{name_filter}%", page_size, page * page_size),
            OrganizationAccount,
        )

    def get_orgs_for_account(self, account_id: UUID) -> List[OrganizationAccount]:
        query = """
        SELECT 
          a.email AS username,
          oa.account_id,
          oa.organization_id,
          oa.role_name
        FROM account AS a 
        INNER JOIN organization_account AS oa ON a.id = oa.account_id 
        WHERE a.id = %s
        """
        return self.db.execute_query_conversion(
            query, (account_id,), OrganizationAccount
        )

    def upsert_account_orgs(
        self, accounts: List[UpsertAccountOrganization]
    ) -> List[UpsertAccountOrganization]:
        query = """
        INSERT INTO organization_account (organization_id, account_id, role_name)
        VALUES {template}
        ON CONFLICT (organization_id, account_id)
          DO UPDATE 
            SET role_name = EXCLUDED.role_name
        RETURNING organization_id, account_id, role_name
        """
        return self.db.bulk_update(accounts, query, ("organization_id", "account_id"))

    def remove_account_org(self, org_id: UUID, account_id: UUID):
        query = """
        DELETE FROM organization_account
        WHERE organization_id = %s AND account_id = %s
        """
        self.db.execute_query_without_fetch(query, (org_id, account_id))

    def get_account_orgs(
        self, account_orgs: List[OrganizationAccountId]
    ) -> List[OrganizationAccount]:
        query = """
        WITH queried_accounts(organization_id, account_id) AS (
          VALUES {template}
        )
        SELECT DISTINCT ON (account.id, organization_account.organization_id, organization_account.role_name)
          account.email AS username, 
          organization_account.account_id, 
          organization_account.organization_id,
          organization_account.role_name
        FROM organization_account
        -- filter the orgs with a join
        INNER JOIN queried_accounts
          ON queried_accounts.organization_id = organization_account.organization_id
          AND queried_accounts.account_id = organization_account.account_id
        INNER JOIN account ON account.id = organization_account.account_id
        """
        return self.db.bulk_query(account_orgs, OrganizationAccount, query)

    def get_usernames_by_org(
        self, name_filter: str, org_id: UUID, page: int, page_size: int
    ) -> List[Username]:
        query = """
        SELECT email AS username, id AS account_id
        FROM account
        INNER JOIN organization_account oa ON account.id = oa.account_id
        WHERE email ILIKE %s ESCAPE '='
          AND organization_id = %s
        ORDER BY LOWER(email)
        LIMIT %s
        OFFSET %s
        """
        name_filter = self.db.unsafe_escape_like_string(
            name_filter, escape_character="="
        )
        return self.db.execute_query_conversion(
            query, (f"%{name_filter}%", org_id, page_size, page_size * page), Username
        )



from enum import Enum
from typing import FrozenSet, Set, Tuple
from uuid import UUID

from beaker.cache import cache_region
from hou_flask_psycopg2 import Psycopg2Utils

from ...cache import LOCAL_LONG_TERM


__all__ = ["Permission", "PermissionDAO", "Role"]


class Permission(Enum):
    create_org = "organization:create"
    update_org = "organization:update"
    update_org_members = "organization:update:members"
    get_all_orgs = "organization:retrieve-all"
    get_org = "organization:retrieve"

    # Feature Flags
    get_feature_flags = "feature-flag:retrieve"
    create_feature_flag = "feature-flag:create"
    delete_feature_flag = "feature-flag:delete"
    update_organization_feature_variant = "feature-flag:organization:set-variant"


class Role(Enum):
    superuser = "superuser"
    default = "default"
    no_role = None


class PermissionDAO:
    db: Psycopg2Utils

    def __init__(self, db: Psycopg2Utils):
        self.db = db

    def get_account_role(self, account_id: UUID) -> str:
        query = """
        SELECT role_name
        FROM account
        WHERE id = %s
        """
        role_name, = self.db.execute_query(query, (account_id,), expect_one=True)
        return role_name

    @cache_region(LOCAL_LONG_TERM, "get_role_permissions")
    def get_role_permissions(self, role: str) -> FrozenSet[str]:
        if role == Role.no_role.value:
            return frozenset()
        query = """
        SELECT role_permission.permission_name
        FROM role_permission
        WHERE role_permission.role_name = %s
        """
        rows = self.db.execute_query(query, (role,))
        return frozenset(row.permission_name for row in rows)

    def get_account_roles_for_org(
        self, account_id: UUID, org_id: UUID
    ) -> Tuple[str, str]:
        query = """
        SELECT 
          account.role_name AS account_role,
          oa.role_name AS org_role
        FROM account
        LEFT JOIN organization_account oa 
          ON account.id = oa.account_id
          AND organization_id = %s
        WHERE account.id = %s
        """
        row = self.db.execute_query(query, (org_id, account_id), expect_one=True)
        return row.account_role, row.org_role

    def get_account_roles_for_department(
        self, account_id: UUID, department_id: UUID
    ) -> Set[str]:
        query = """
        SELECT role_name
        FROM account
        WHERE id = %(account_id)s

        UNION ALL

        SELECT role_name
        FROM department
        INNER JOIN organization_account
          ON department.organization_id = organization_account.organization_id
        WHERE department.id = %(department_id)s
          AND organization_account.account_id = %(account_id)s

        UNION ALL

        SELECT role_name
        FROM department
        INNER JOIN department parent
          ON parent.parent_path @> department.parent_path
        INNER JOIN department_account da 
          ON parent.id = da.department_id
          AND %(account_id)s = da.account_id
        WHERE department.id = %(department_id)s
        """
        params = {"account_id": account_id, "department_id": department_id}
        return {row[0] for row in self.db.execute_query(query, params)}



from typing import List
from uuid import UUID

from hou_flask_psycopg2 import Psycopg2Utils

from houflask.auth.daos.authentication_dao import Username, Account


class AccountDAO:
    db: Psycopg2Utils

    def __init__(self, db: Psycopg2Utils):
        self.db = db

    def get_username(self, user_id: UUID) -> Username:
        query = """
        SELECT email AS username, id AS account_id
        FROM account
        WHERE id = %s
        """
        return self.db.execute_query_conversion(
            query, (user_id,), Username, expect_one=True
        )

    def get_usernames(
        self, name_filter: str, page: int, page_size: int
    ) -> List[Username]:
        query = """
        SELECT email AS username, id AS account_id
        FROM account
        WHERE email ILIKE %s ESCAPE '='
        ORDER BY LOWER(email)
        LIMIT %s
        OFFSET %s
        """
        name_filter = self.db.unsafe_escape_like_string(
            name_filter, escape_character="="
        )
        return self.db.execute_query_conversion(
            query, (f"%{name_filter}%", page_size, page_size * page), Username
        )

    def get_account_info(self, account_id: UUID) -> Account:
        query = """
        SELECT email AS username, id
        FROM account
        WHERE id = %s
        """
        return self.db.execute_query_conversion(
            query, (account_id,), Account, expect_one=True
        )



import pytest

from oss_auth.database import DB
from oss_auth.exceptions import NotLoggedInException
from oss_auth.utils.authorization import get_current_account

from houflask.auth.services.authorization_service import AUTH_COOKIE
from oss_auth.views.authentication import AUTH_COOKIE

from ..utils.conditionals import (
    skip_unless_cognito_tests_enabled,
    skip_unless_create_cognito_tests_enabled,
)
from ..utils.factories.account import TokenFactory
from ..utils.factories.base_factory import random_email, random_string
from .base import FunctionalBase


class RegisterTest(FunctionalBase):
    url = "/v1/authentication/register"

    def test_when_password_mismatch__raises_400_error(self, test_app):
        email = random_email()
        password = "P@ssw0rd"
        self.assert_post(
            test_app,
            {"email": email, "password": password, "confirm_password": "blahblah"},
            400,
        )

    @skip_unless_create_cognito_tests_enabled
    def test_when_passwords_match__creates_user(self, test_app):
        email = f"{random_string()}@{random_string()}.com"
        password = "P@ssw0rd"
        self.assert_post(
            test_app,
            {"email": email, "password": password, "confirm_password": password},
            201,
        )
        DB.execute_query(
            "SELECT 1 FROM account WHERE email = %s", (email,), expect_one=True
        )


class LoginTest(FunctionalBase):
    url = "/v1/authentication/login"

    @skip_unless_cognito_tests_enabled
    def test_when_valid_login__sets_token(self, test_app):
        email = "test@timmartin.me"
        password = "P@ssw0rd"
        resp = self.assert_post(
            test_app, {"username": email, "password": password, "remember": True}, 201
        )
        assert resp.headers["Set-Cookie"].startswith(f"{AUTH_COOKIE}=")


class GetCurrentAccountTest:
    def test_when_no_token(self, test_app):
        with test_app.app.test_request_context():
            with pytest.raises(NotLoggedInException):
                get_current_account()

    def test_when_token_not_exists(self, test_app):
        with test_app.app.test_request_context(
            headers={"Cookie": "{}=blah".format(AUTH_COOKIE)}
        ):
            with pytest.raises(NotLoggedInException):
                get_current_account()

    def test_when_token_valid__returns_id(self, test_app):
        token = TokenFactory()
        cookie = "{}={}".format(AUTH_COOKIE, token.token)
        with test_app.app.test_request_context(headers={"Cookie": cookie}):
            account_id = get_current_account()
            assert account_id == token.account_id


class LogoutTest(FunctionalBase):
    url = "/v1/authentication/logout"

    def test_when_logged_in__returns_200_and_deletes_cookie(
        self, test_app, superuser_token
    ):
        resp = self.assert_get(test_app, {}, 200, token=superuser_token)

        rows = DB.execute_query(
            "SELECT TRUE FROM oss_auth.token WHERE token = %s", (superuser_token.token,)
        )
        assert rows == []
        assert "login_url" in resp.json
        assert "auth-token=;" in resp.headers["Set-Cookie"]

    def test_when_not_logged_in__returns_200(self, test_app, superuser_token):
        self.assert_get(test_app, {}, 200)


class GetCSRFTokenTest(FunctionalBase):
    url = "/v1/authentication/csrf_token"

    def test_when_valid__returns_token(self, test_app):
        resp = self.assert_get(test_app, {}, 200)
        assert "csrf_token" in resp.json



from uuid import uuid4

import pytest
from hou_flask_psycopg2 import CustomJSON
from psycopg2.errors import ForeignKeyViolation

from oss_auth.daos.feature_flag_dao import (
    FeatureFlagDAO,
    FeatureVariant,
    NewFeatureVariant,
)
from oss_auth.database import DB
from oss_auth.exceptions import DuplicateFeatureFlagException
from tests.utils.factories import OrganizationFactory
from tests.utils.factories.base_factory import random_string
from tests.utils.factories.feature_flag import (
    FeatureFlagFactory,
    FeatureVariantFactory,
    FeatureVariantOrganizationFactory,
)


class CreateFeatureFlagTest:
    def test_when_valid__returns_feature_flag(self):
        name = random_string()
        FeatureFlagDAO.create_feature_flag(name, False)
        resp = FeatureFlagDAO.get_feature_flags()
        assert name in [ff.name for ff in resp]

    def test_when_duplicate_feature_flag_name__raises_exception(self):
        name = random_string()
        FeatureFlagDAO.create_feature_flag(name, False)
        with pytest.raises(DuplicateFeatureFlagException):
            FeatureFlagDAO.create_feature_flag(name, True)


class CreateFeatureVariantsTest:
    def test_when_empty__returns_empty(self):
        resp = FeatureFlagDAO.create_feature_variants([])
        assert resp == []

    def test_when_valid__returns_feature_variants(self):
        feature_flag = FeatureFlagFactory()
        variants = [
            NewFeatureVariant(feature_flag.name, True),
            NewFeatureVariant(feature_flag.name, False),
        ]
        resp = FeatureFlagDAO.create_feature_variants(variants)
        assert len(resp) == 2
        query = "SELECT COUNT(*) FROM feature_variant WHERE flag_name = %s"
        insert_count = DB.execute_query(query, (feature_flag.name,), expect_one=True)[0]
        assert insert_count == 2


class CreateFeatureVariantOrganizationTest:
    def test_when_variant_not_exists__raises_exception(self):
        variant = FeatureVariant(flag_name="not-real", variant=True, id=uuid4())
        org = OrganizationFactory()
        with pytest.raises(ForeignKeyViolation):
            FeatureFlagDAO.create_feature_variant_organization(
                variant.flag_name, variant.id, [org.id]
            )

    def test_when_empty__returns_empty(self):
        variant = FeatureVariantFactory()
        resp = FeatureFlagDAO.create_feature_variant_organization(
            variant.flag_name, variant.id, []
        )
        assert resp == []

    def test_when_valid__creates_variants_for_org(self):
        variant = FeatureVariantFactory()
        org_ids = [org.id for org in OrganizationFactory.build_batch(2)]
        resp = FeatureFlagDAO.create_feature_variant_organization(
            variant.flag_name, variant.id, org_ids
        )
        assert len(resp) == 2


class DeleteFeatureVariantOrganizationTest:
    def test_when_exists__deletes_organization(self, db_helper):
        variant = FeatureVariantOrganizationFactory()
        FeatureFlagDAO.delete_feature_variant_organization(
            variant.organization_id, variant.flag_name
        )
        db_helper.assert_database_has(
            "feature_variant_organization",
            {
                "organization_id": variant.organization_id,
                "flag_name": variant.flag_name,
            },
            0,
        )

    def test_when_not_exists__no_error(self):
        FeatureFlagDAO.delete_feature_variant_organization(uuid4(), "not-real")


class DeleteFeatureVariantsTest:
    def test_when_empty__safely_returns(self):
        FeatureFlagDAO.delete_feature_variants([])

    def test_when_valid__deletes_all(self):
        variants = FeatureVariantFactory.build_batch(2)
        ids = [var.id for var in variants]
        FeatureFlagDAO.delete_feature_variants(ids)
        count = DB.execute_query(
            "SELECT COUNT(*) FROM feature_variant WHERE id IN %s",
            (tuple(ids),),
            expect_one=True,
        )[0]
        assert count == 0


class GetFeatureFlagsTest:
    def test_when_valid__returns_all_feature_flags(self):
        feature_flags = FeatureFlagFactory.build_batch(2)
        resp = FeatureFlagDAO.get_feature_flags()
        assert {ff.name for ff in feature_flags}.issubset({row.name for row in resp})


class GetOrgVariantTest:
    def test_when_no_org_variant__returns_default(self):
        feature_flag = FeatureFlagFactory()
        org = OrganizationFactory()
        resp = FeatureFlagDAO.get_org_variant(feature_flag.name, org.id)
        assert resp == feature_flag.default_value

    def test_when_org_variant__returns_org_variant(self):
        org_variant = FeatureVariantOrganizationFactory(
            feature_variant__variant=CustomJSON(123)
        )
        resp = FeatureFlagDAO.get_org_variant(
            org_variant.flag_name, org_variant.organization_id
        )
        assert resp == 123



from uuid import uuid4

import pytest
from psycopg2 import IntegrityError

from hou_flask_psycopg2 import SQLNotFoundException
from oss_auth.daos.organization_dao import Organization, OrganizationDAO
from oss_auth.database import DB
from oss_auth.exceptions import OrganizationNameUnavailable, OrganizationNotFound
from tests.utils.factories.organization import OrganizationFactory


class CreateTest:
    def test_when_org_name_exists__raises_exception(self):
        OrganizationDAO.create("blah", False)
        with pytest.raises(OrganizationNameUnavailable):
            OrganizationDAO.create("blah", True)

    def test_when_valid__returns_org(self):
        actual = OrganizationDAO.create("blah", True)
        assert actual is not None
        expected = Organization(id=actual.id, name="blah", active=True)
        assert actual == expected

    def test_when_name_null__raises_exception(self):
        with pytest.raises(IntegrityError):
            OrganizationDAO.create(None, True)


class UpdateTest:
    def test_when_org_not_exists__raises_exception(self):
        with pytest.raises(OrganizationNotFound):
            OrganizationDAO.update(uuid4(), "blag", True)

    def test_when_new_name_exists__raises_exception(self):
        existing = OrganizationFactory.build()
        org = OrganizationFactory.build()
        with pytest.raises(OrganizationNameUnavailable):
            OrganizationDAO.update(org.id, existing.name, True)

    def test_when_name_null__raises_integrity_error(self):
        org = OrganizationFactory.build()
        with pytest.raises(IntegrityError):
            OrganizationDAO.update(org.id, None, True)

    def test_when_valid__returns_updated_org(self):
        org = OrganizationFactory.build()
        resp = OrganizationDAO.update(org.id, "newname", False)
        assert resp.name == "newname"
        assert resp.active is False

        query = "SELECT 1 FROM oss_auth.organization WHERE name = 'newname' AND active IS FALSE"
        resp = DB.execute_query(query, {})
        assert len(resp) == 1


class GetAllOrgsTest:
    def test_when_no_filter__returns_all(self):
        orgs = OrganizationFactory.build_batch(3)
        resp = OrganizationDAO.get_all_orgs()
        expected_ids = {org.id for org in orgs}
        actual_ids = {org.id for org in resp}
        assert expected_ids.issubset(actual_ids)

    def test_when_filter__only_returns_matches(self):
        org1 = OrganizationFactory(name="forreal")
        OrganizationFactory(name="another")

        resp = OrganizationDAO.get_all_orgs(name_filter="orrea")
        assert len(resp) == 1
        assert resp[0].id == org1.id

    def test_when_page_and_page_size__returns_offset(self):
        org1 = OrganizationFactory(name="forreals")
        OrganizationFactory(name="another")
        resp = OrganizationDAO.get_all_orgs(page=1, page_size=1)
        assert len(resp) == 1
        assert resp[0].id == org1.id

    def test_when_valid__returns_sorted_case_insensitvie(self):
        OrganizationFactory(name="ForReals")
        OrganizationFactory(name="blah")
        OrganizationFactory(name="Another")

        resp = OrganizationDAO.get_all_orgs()
        for i in range(len(resp) - 1):
            assert resp[i].name.lower() < resp[i + 1].name.lower()


class GetOrgTest:
    def test_when_org_not_exists__raises_exception(self):
        with pytest.raises(SQLNotFoundException):
            OrganizationDAO.get_org(uuid4())

    def test_when_org_exists__returns_org(self):
        expected = OrganizationFactory.build()
        actual = OrganizationDAO.get_org(expected.id)
        assert actual.id == expected.id


class GetOrgsTest:
    def test_when_all_found__returns_orgs(self):
        orgs = OrganizationFactory.build_batch(2)
        org_ids = {org.id for org in orgs}

        resp = OrganizationDAO.get_orgs(org_ids)

        assert len(resp) == 2

    def test_when_some_not_found__raises_sql_not_found_exception(self):
        org = OrganizationFactory()
        org_ids = {org.id, uuid4()}

        with pytest.raises(SQLNotFoundException):
            OrganizationDAO.get_orgs(org_ids)



from functools import wraps
from typing import AbstractSet, Any, Callable, Dict, Set, Tuple, Optional
from uuid import UUID

from flask import g, request
from hou_flask_psycopg2 import Psycopg2Utils

from houflask.auth.daos.permission_dao import Permission, PermissionDAO
from houflask.exceptions import (
    MissingPermissionsException,
    NotLoggedInException,
    ParameterNotPresentException,
)
from houflask.auth.services.authentication_service import AuthenticationService

AUTH_COOKIE = "auth-token"
__all__ = ["AUTH_COOKIE", "AuthorizationService"]


class AuthorizationService:
    permission_dao: PermissionDAO
    authentication_service: AuthenticationService

    def __init__(self, db: Optional[Psycopg2Utils] = None):
        if db:
            self.init_db(db)

    def init_db(self, db: Psycopg2Utils):
        self.permission_dao = PermissionDAO(db)
        self.authentication_service = AuthenticationService(db)

    def require_global_permissions(
        self, *permissions: Permission
    ) -> Callable[[Callable], Callable]:
        required_permissions = frozenset(permissions)

        def _decorator(func: Callable) -> Callable:
            @wraps(func)
            def _wrapper(*args: Tuple[Any], **kwargs: Dict[Any, Any]) -> Any:
                account_id = self.get_current_account()
                account_role = self.permission_dao.get_account_role(account_id)
                user_permissions = self.permission_dao.get_role_permissions(
                    account_role
                )
                self._require_permissions(required_permissions, user_permissions)
                return func(*args, **kwargs)

            # pylint: disable=protected-access
            _wrapper._required_permissions = required_permissions  # type: ignore
            return _wrapper

        return _decorator

    def require_logged_in(self, func):
        @wraps(func)
        def _wrapper(*args, **kwargs):
            self.get_current_account()
            return func(*args, **kwargs)

        return _wrapper

    @staticmethod
    def _require_permissions(
        required_permissions: AbstractSet[Permission],
        actual_permissions: AbstractSet[str],
    ) -> None:
        missing_permissions = {
            p.value for p in required_permissions
        } - actual_permissions
        if missing_permissions:
            raise MissingPermissionsException(missing_permissions)

    def get_current_account(self) -> UUID:
        # pylint: disable=protected-access
        if not hasattr(g, "_current_account"):
            token = request.cookies.get(AUTH_COOKIE)
            if not token:
                raise NotLoggedInException
            g._current_account = self.authentication_service.get_account_by_token(token)
        return g._current_account

    def _verify_role_permissions(
        self,
        role_names: AbstractSet[str],
        required_permissions: AbstractSet[Permission],
    ) -> None:
        assert required_permissions, "Override permissions must be provided"
        all_permissions = set()  # type: Set[Permission]
        for role_name in role_names:
            all_permissions |= self.permission_dao.get_role_permissions(role_name)
        self._require_permissions(required_permissions, all_permissions)

    def _require_permissions_helper(
        self,
        *required_permissions: Permission,
        kwarg_name: str = None,
        get_roles_func: Callable = None,
    ):
        def _require_account_org_permissions_decorator(func: Callable) -> Callable:
            @wraps(func)
            def _require_account_org_permissions_wrapper(*args, **kwargs):
                if kwarg_name not in kwargs:
                    raise ParameterNotPresentException(
                        f"{func.__module__}.{func.__name__}", kwarg_name
                    )
                role_names = get_roles_func()(
                    self.get_current_account(), kwargs[kwarg_name]
                )
                self._verify_role_permissions(
                    set(role_names), set(required_permissions)
                )
                return func(*args, **kwargs)

            return _require_account_org_permissions_wrapper

        return _require_account_org_permissions_decorator

    def require_org_permissions(self, *required_permissions):
        return self._require_permissions_helper(
            *required_permissions,
            kwarg_name="organization_id",
            get_roles_func=lambda: self.permission_dao.get_account_roles_for_org,
        )



from typing import List
from uuid import UUID

from ..daos.organization_account_dao import (
    OrganizationAccountDAO,
    OrganizationAccount,
    OrganizationAccountId,
    UpsertAccountOrganization,
)


class OrganizationAccountService:
    organization_account_dao: OrganizationAccountDAO

    def __init__(self, organization_account_dao: OrganizationAccountDAO):
        self.organization_account_dao = organization_account_dao

    def get_accounts_for_org(
        self,
        organization_id: UUID,
        name_filter: str = "",
        page: int = 0,
        page_size: int = 100,
    ) -> List[OrganizationAccount]:
        return self.organization_account_dao.get_accounts_for_org(
            organization_id, name_filter, page, page_size
        )

    def get_orgs_for_account(self, account_id: UUID) -> List[OrganizationAccount]:
        return self.organization_account_dao.get_orgs_for_account(account_id)

    def upsert_account_orgs(
        self, accounts: List[UpsertAccountOrganization]
    ) -> List[OrganizationAccount]:
        created_accounts = self.organization_account_dao.upsert_account_orgs(accounts)
        org_account_ids = [
            OrganizationAccountId(
                organization_id=account.organization_id, account_id=account.account_id
            )
            for account in created_accounts
        ]
        return self.organization_account_dao.get_account_orgs(org_account_ids)

    def remove_account_org(self, org_id: UUID, account_id: UUID) -> None:
        self.organization_account_dao.remove_account_org(org_id, account_id)



from dataclasses import dataclass
from typing import List, Set
from uuid import UUID

from hou_flask_psycopg2 import Psycopg2Utils
from psycopg2 import IntegrityError
from psycopg2.errorcodes import UNIQUE_VIOLATION
from psycopg2.extras import DictCursor

from ...exceptions import OrganizationNameUnavailable, OrganizationNotFound


@dataclass
class Organization:
    name: str
    id: UUID
    active: bool


class OrganizationDAO:
    db: Psycopg2Utils

    def __init__(self, db: Psycopg2Utils):
        self.db = db

    def create(self, name: str, active: bool = True) -> Organization:
        query = """
        INSERT INTO organization (name, active)
        VALUES (%(name)s, %(active)s)
        RETURNING id, name, active
        """
        try:
            rows = self.db.execute_query(
                query, {"name": name, "active": active}, factory=DictCursor
            )
        except IntegrityError as exc:
            if exc.pgcode == UNIQUE_VIOLATION:
                raise OrganizationNameUnavailable(
                    f'The organization name "{name}" is unavailable'
                ) from exc
            raise exc
        # TODO: Remove type ignore when mypy actually supports dataclasses
        return Organization(**rows[0])  # type: ignore

    def update(self, id_: UUID, name: str, active: bool) -> Organization:
        query = """
        UPDATE organization
        SET name = %(name)s,
            active = %(active)s
        WHERE id = %(id)s
        RETURNING id, name, active
        """
        try:
            rows = self.db.execute_query(
                query, {"id": id_, "name": name, "active": active}, factory=DictCursor
            )
        except IntegrityError as exc:
            if exc.pgcode == UNIQUE_VIOLATION:
                raise OrganizationNameUnavailable(
                    f'The organization name "{name}" is unavailable'
                ) from exc
            raise exc
        else:
            if not rows:
                raise OrganizationNotFound(id_)
            # TODO: Remove type ignore when mypy actually supports dataclasses
            return Organization(**rows[0])  # type: ignore

    def get_orgs_for_account(self, account_id: UUID) -> List[Organization]:
        query = """
        SELECT name, id, active
        FROM organization AS o
        INNER JOIN organization_account AS oa
          ON oa.organization_id = o.id
        WHERE oa.account_id = %s
        """
        return self.db.execute_query_conversion(query, (account_id,), Organization)

    def get_all_orgs(
        self, page: int = 0, page_size: int = 100, name_filter: str = ""
    ) -> List[Organization]:
        """
        Get all orgs with a case insensitive match to the filter.
        By default returns all orgs.  Orgs, are sorted by name case
        insensitively. Pages are 0 indexed.
        """
        query = """
        SELECT name, id, active
        FROM organization
        WHERE name ILIKE %(filter)s ESCAPE '='
        ORDER BY LOWER(name)
        LIMIT %(page_size)s
        OFFSET %(offset)s
        """
        name_filter = self.db.unsafe_escape_like_string(name_filter)
        params = {
            "filter": f"%{name_filter}%",
            "page_size": page_size,
            "offset": page * page_size,
        }
        return self.db.execute_query_conversion(query, params, Organization)

    def get_orgs(self, org_ids: Set[UUID]) -> List[Organization]:
        if not org_ids:
            return []
        query = """
        SELECT name, id, active
        FROM organization
        WHERE id IN %s
        """
        rows = self.db.execute_query_conversion(query, (tuple(org_ids),), Organization)
        return self.db.verify_row_count(rows, len(org_ids))

    def get_org(self, organization_id: UUID) -> Organization:
        return self.get_orgs({organization_id})[0]

    def org_name_exists(self, name: str) -> bool:
        query = "SELECT 1 FROM organization WHERE name = %(name)s"
        return len(self.db.execute_query(query, {"name": name})) > 0



import dataclasses
from functools import wraps
from typing import Dict, Iterable, Union, Callable
from uuid import UUID

import rapidjson
from flask import Response


def json_response(data: Union[Dict, Iterable], status=200):
    return Response(rapidjson.dumps(data), status, content_type="application/json")


def empty_response():
    return Response(status=204)


def dataclass_json_response(data, status: int = 200):
    """
    Turns a list or single instance of dataclasses into a json response
    """
    try:
        return json_response(dataclasses.asdict(data), status)
    except TypeError:
        return json_response((dataclasses.asdict(item) for item in data), status)


def uuid_converter(key_words: Iterable[str]) -> Callable:
    def _uuid_converter_decorator(func: Callable) -> Callable:
        @wraps(func)
        def _uuid_converter_wrapper(*args, **kwargs):
            for key_word in key_words:
                if key_word in kwargs:
                    kwargs[key_word] = UUID(kwargs[key_word])
            return func(*args, **kwargs)

        return _uuid_converter_wrapper

    return _uuid_converter_decorator



import copy
import random
from collections import namedtuple
from enum import Enum
from string import ascii_letters
from typing import Dict, Iterable, List

from factory import Factory
from hou_flask_psycopg2 import Psycopg2Utils
from psycopg2.extras import DictCursor
from psycopg2.sql import SQL, Identifier


def random_string(length=10):
    return "".join(random.choice(ascii_letters) for _ in range(length))


def random_email(length=10):
    # This format doesn't cause AWS to censure us
    # They accept these but they don't actually send an email I believe
    return "success+{}@simulator.amazonses.com".format(random_string(length))


class Model:
    def __init__(self, table_name: str, db: Psycopg2Utils):
        self.table = table_name
        self.db = db

    def __call__(self, **kwargs):
        model_class = namedtuple(self.table, kwargs.keys())
        return model_class(**kwargs)


class EmptyRelationship:
    def __getattr__(self, item):
        return None


_RELATIONSHIP_TYPES = (tuple, EmptyRelationship, Enum)


class BaseFactory(Factory):
    # pylint: disable=arguments-differ
    @classmethod
    def _build(cls, model_class, **kwargs):
        relationships = {
            k: v for k, v in kwargs.items() if isinstance(v, _RELATIONSHIP_TYPES)
        }
        real_values = {
            k: v for k, v in kwargs.items() if not isinstance(v, _RELATIONSHIP_TYPES)
        }
        keys = ["{{{}}}".format(k) for k in real_values.keys()]
        values = ", ".join(["%({})s".format(k) for k in real_values.keys()])
        identifiers = {k: Identifier(k) for k in real_values.keys()}
        query = SQL(
            """
        INSERT INTO {{table_name}} ({keys})
        VALUES ({values})
        RETURNING *
        """.format(
                keys=", ".join(keys), values=values
            )
        ).format(table_name=Identifier(model_class.table), **identifiers)
        resp = dict(
            model_class.db.execute_query(query, real_values, factory=DictCursor)[0]
        )
        resp.update(**relationships)
        model = model_class(**resp)
        return model

    @classmethod
    def _create(cls, model_class, *args, **kwargs):
        return cls._build(model_class, *args, **kwargs)

    @classmethod
    def build_batch(cls, *args, **kwargs):
        return super(BaseFactory, cls).build_batch(*args, **kwargs)

    @classmethod
    def build_distinct_batch(
        cls, distinct_kwargs: Iterable[Dict], **common_kwargs
    ) -> List:
        models = []
        for kwargs in distinct_kwargs:
            kwargs = copy.deepcopy(kwargs)
            kwargs.update(common_kwargs)
            models.append(cls.build(**kwargs))
        return models



from uuid import uuid4

import rapidjson

from tests.functional_tests.base import FunctionalBase
from tests.utils.factories.base_factory import random_string
from tests.utils.factories.feature_flag import FeatureFlagFactory, FeatureVariantFactory


class GetFeatureFlagsTest(FunctionalBase):
    url = "/v1/flag/feature_flag"

    def test_when_valid__returns_feature_flags_and_variants(
        self, test_app, superuser_token
    ):
        feature1 = FeatureFlagFactory()
        feature2 = FeatureVariantFactory()
        resp = self.assert_get(test_app, {}, 200, token=superuser_token)
        flags = {flag["name"] for flag in resp.json["feature_flags"]}
        assert feature1.name in flags
        assert feature2.flag_name in flags
        variants = {
            (variant["flag_name"], variant["variant"])
            for variant in resp.json["variants"]
        }
        assert (feature2.flag_name, feature2.variant) in variants


class CreateFeatureFlagTest(FunctionalBase):
    url = "/v1/flag/feature_flag"

    def test_when_valid__creates_feature_flag(
        self, test_app, superuser_token, db_helper
    ):
        name = random_string()
        body = {"name": name, "default_value": True}
        resp = self.assert_post(test_app, body, 201, token=superuser_token)
        assert resp.json == body
        db_helper.assert_database_has(
            "feature_flag", {"name": name, "default_value": rapidjson.dumps(True)}
        )

    def test_when_duplicate_name__returns_400(self, test_app, superuser_token):
        flag = FeatureFlagFactory()
        body = {"name": flag.name, "default_value": True}
        self.assert_post(test_app, body, 400, token=superuser_token)


class CreateVariantTest(FunctionalBase):
    url = "/v1/flag/feature_variant"

    def test_when_valid__creates_variant(self, test_app, superuser_token):
        flag = FeatureFlagFactory()
        body = {"flag_name": flag.name, "variant": True}
        resp = self.assert_post(test_app, body, 201, flag.name, token=superuser_token)
        assert body.items() <= resp.json.items()


class DeleteVariantTest(FunctionalBase):
    url = "/v1/flag/feature_variant/{}"

    def test_when_valid__deletes_variant(self, test_app, superuser_token, db_helper):
        variant = FeatureVariantFactory()
        self.assert_delete(test_app, {}, 204, variant.id, token=superuser_token)
        db_helper.assert_database_has("feature_variant", {"id": variant.id}, 0)

    def test_when_does_not_exists__returns_successful_response(
        self, test_app, superuser_token
    ):
        self.assert_delete(test_app, {}, 204, uuid4(), token=superuser_token)



import rapidjson
from flask_webtest import TestApp

from houflask.auth.services.authorization_service import AUTH_COOKIE


class FunctionalBase:
    @classmethod
    def assert_post(
        cls, test_app: TestApp, params, status_code, *url_params, token=None, query=None
    ):
        url = cls._request(test_app, *url_params, token=token, query=query)
        resp = test_app.post(
            url,
            rapidjson.dumps(params),
            content_type="application/json",
            expect_errors=True,
        )
        return cls._assert_resp(resp, status_code)

    @classmethod
    def assert_get(
        cls, test_app: TestApp, params, status_code, *url_params, token=None, query=None
    ):
        url = cls._request(test_app, *url_params, token=token, query=query)
        resp = test_app.get(url, params, expect_errors=True)
        return cls._assert_resp(resp, status_code)

    @classmethod
    def assert_put(
        cls, test_app: TestApp, params, status_code, *url_params, token=None, query=None
    ):
        url = cls._request(test_app, *url_params, token=token, query=query)
        resp = test_app.put(
            url,
            rapidjson.dumps(params),
            content_type="application/json",
            expect_errors=True,
        )
        return cls._assert_resp(resp, status_code)

    @classmethod
    def assert_delete(
        cls, test_app: TestApp, params, status_code, *url_params, token=None, query=None
    ):
        url = cls._request(test_app, *url_params, token=token, query=query)
        resp = test_app.delete(
            url,
            rapidjson.dumps(params),
            content_type="application/json",
            expect_errors=True,
        )
        return cls._assert_resp(resp, status_code)

    @classmethod
    def _request(cls, test_app: TestApp, *url_params, token=None, query: str = None):
        if token:
            test_app.set_cookie(AUTH_COOKIE, token.token)
        url = cls._build_url(*url_params, query=query)
        return url

    @classmethod
    def _assert_resp(cls, resp, expected_status_code):
        assert (
            resp.status_code == expected_status_code
        ), f"{resp.status_code} != {expected_status_code}. body: {resp.body}"
        return resp

    @classmethod
    def _build_url(cls, *url_params, query=None):
        url = cls.url.format(*url_params)
        return url if not query else f"{url}?{query}"



import logging
import re
from datetime import date, datetime
from uuid import UUID

from connexion import App as ConnexionApp
from connexion.decorators.response import ResponseValidator
from connexion.decorators.validation import ParameterValidator, RequestBodyValidator
from connexion.json_schema import Draft4RequestValidator, Draft4ResponseValidator
from connexion.utils import TYPE_MAP, is_null, is_nullable
from flask import Flask
from jsonschema import draft4_format_checker, validators
from ultra_config import GlobalConfig
from werkzeug.routing import BaseConverter, ValidationError

__all__ = ["FlaskConnexionExtension"]
LOG = logging.getLogger(__name__)
UUID_RE = re.compile(r"^[0-9a-f]{8}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{12}$")


class FlaskConnexionExtension:
    app: Flask = None
    connexion_app: ConnexionApp = None

    def init_app(self, connexion_app: ConnexionApp) -> Flask:
        self.connexion_app = connexion_app
        self.app = self.connexion_app.app
        self._init_url_converters()
        draft4_format_checker.checks(format="uuid")(self.verify_uuid)
        draft4_format_checker.checks(format="date-time")(self.verify_datetime)
        draft4_format_checker.checks(format="date")(self.verify_date)
        return self.app

    @GlobalConfig.inject(
        strict_validation="SCHEMA_VALIDATION_STRICT",
        validate_responses="SCHEMA_VALIDATION_VALIDATE_RESPONSES",
    )
    def add_api(
        self,
        swagger_file: str,
        strict_validation: bool = True,
        validate_responses: bool = False,
        **kwargs
    ):
        return self.connexion_app.add_api(
            swagger_file,
            validator_map=self.monkeypatch_jsonschema_validation(),
            strict_validation=strict_validation,
            validate_responses=validate_responses,
            **kwargs
        )

    @staticmethod
    def verify_uuid(value):
        return isinstance(value, UUID) or value is None

    @staticmethod
    def verify_datetime(value):
        return isinstance(value, datetime)

    @staticmethod
    def verify_date(value):
        return isinstance(value, date)

    @staticmethod
    def monkeypatch_jsonschema_validation():
        return {
            "body": _UUIDRequestBodyValidator,
            "parameter": _UUIDParameterValidator,
            "response": _UUIDResponseBodyValidator,
        }

    def _init_url_converters(self):
        self.app.url_map.converters["UUID"] = _UUIDConverter


def _custom_json_schema_extend(validator, _validators, version=None, types=None):
    all_validators = dict(validator.VALIDATORS)
    all_validators.update(_validators)
    all_types = dict(validator.DEFAULT_TYPES)
    types = types or {}
    all_types.update(types)
    return validators.create(
        meta_schema=validator.META_SCHEMA,
        validators=all_validators,
        version=version,
        default_types=all_types,
    )


class _UUIDConverter(BaseConverter):
    """
    UUID converter for the Werkzeug routing system.

    Shamelessly borrowed from https://github.com/wbolster/flask-uuid
    """

    def __init__(self, map_, strict=True):
        super(_UUIDConverter, self).__init__(map_)
        self.strict = strict

    def to_python(self, value):
        if self.strict and not UUID_RE.match(value):
            raise ValidationError()

        try:
            return UUID(value)
        except ValueError:
            raise ValidationError()

    def to_url(self, value):
        return str(value)


def _build_uuid_validator_class(validator):
    extra_types = {"string": (*validators.str_types, UUID, datetime, date)}
    TYPE_MAP.update({"string": lambda value: value})
    return _custom_json_schema_extend(validator, {}, types=extra_types)


class _UUIDRequestBodyValidator(RequestBodyValidator):
    def __init__(self, *args, **kwargs):
        super().__init__(
            *args,
            validator=_build_uuid_validator_class(Draft4RequestValidator),
            **kwargs
        )


class _UUIDResponseBodyValidator(ResponseValidator):
    def __init__(self, *args, **kwargs):
        super().__init__(
            *args,
            validator=_build_uuid_validator_class(Draft4ResponseValidator),
            **kwargs
        )


class _UUIDParameterValidator(ParameterValidator):
    # pylint: disable=inconsistent-return-statements
    @staticmethod
    def validate_parameter(parameter_type, value, param, param_name=None):
        if value is not None:
            if is_nullable(param) and is_null(value):
                return
            schema = param.get("schema", {})

            if schema.get("type") == "string" and schema.get("format") == "uuid":
                try:
                    UUID(value)
                except ValueError as exc:
                    return str(exc)
            else:
                return ParameterValidator.validate_parameter(
                    parameter_type, value, param, param_name
                )
        return None



from uuid import uuid4

import pytest
from hou_flask_psycopg2 import SQLNotFoundException
from psycopg2 import DataError, IntegrityError

from oss_auth.daos.account_dao import (
    Account,
    AccountDAO,
    UpsertAccountOrganization,
    Username,
)
from oss_auth.utils.permission import Role
from tests.utils.factories import (
    AccountFactory,
    DepartmentAccountFactory,
    DepartmentFactory,
    OrganizationAccountFactory,
    OrganizationFactory,
)
from tests.utils.factories.base_factory import EmptyRelationship, random_string


class GetOrgsForAccountTest:
    def test_when_account_id_invalid__raises_data_error(self):
        with pytest.raises(DataError):
            AccountDAO.get_orgs_for_account("balh")

    def test_when_valid__returns_all_associations(self):
        account = AccountFactory()
        org_accounts = OrganizationAccountFactory.build_batch(3, account=account)

        resp = AccountDAO.get_orgs_for_account(account.id)

        expected_org_ids = {oa.organization_id for oa in org_accounts}
        actual_org_ids = {row.organization_id for row in resp}
        assert actual_org_ids == expected_org_ids


class UpsertAccountOrgsTest:
    def test_when_multiple_accounts_all_new__inserts_all_rows(self):
        org = OrganizationFactory()
        accounts = AccountFactory.build_batch(3)
        account_rows = [
            UpsertAccountOrganization(
                account_id=account.id,
                organization_id=org.id,
                role_name=Role.default.value,
            )
            for account in accounts
        ]
        resp = AccountDAO.upsert_account_orgs(account_rows)
        assert len(resp) == 3
        actual_accounts = AccountDAO.get_accounts_for_org(org.id)
        self._assert_upserted_accounts_equal_accounts(resp, actual_accounts)

    def test_when_multiple_accounts_all_existing__updates_all_rows(self):
        org = OrganizationFactory()
        accounts = AccountFactory.build_batch(3, role_name=Role.superuser.value)
        account_rows = [
            UpsertAccountOrganization(
                account_id=account.id,
                organization_id=org.id,
                role_name=Role.default.value,
            )
            for account in accounts
        ]
        OrganizationAccountFactory.build_distinct_batch(
            [{"account_id": u.account_id} for u in account_rows],
            organization=org,
            account=EmptyRelationship(),
        )
        resp = AccountDAO.upsert_account_orgs(account_rows)
        assert len(resp) == 3
        actual_accounts = AccountDAO.get_accounts_for_org(org.id)
        self._assert_upserted_accounts_equal_accounts(resp, actual_accounts)

    def test_when_multiple_accounts_some_new_some_existing__updates_and_inserts(self):
        org = OrganizationFactory()
        accounts = AccountFactory.build_batch(3, role_name=Role.superuser.value)
        account_rows = [
            UpsertAccountOrganization(
                account_id=account.id,
                organization_id=org.id,
                role_name=Role.default.value,
            )
            for account in accounts
        ]
        OrganizationAccountFactory.build_distinct_batch(
            [{"account_id": u.account_id} for u in account_rows[1:]],
            organization=org,
            account=EmptyRelationship(),
        )
        resp = AccountDAO.upsert_account_orgs(account_rows)
        assert len(resp) == 3
        actual_accounts = AccountDAO.get_accounts_for_org(org.id)
        self._assert_upserted_accounts_equal_accounts(resp, actual_accounts)

    def test_when_invalid_account_id__raises_sql_not_found_exception(self):
        org = OrganizationFactory()
        with pytest.raises(IntegrityError):
            AccountDAO.upsert_account_orgs(
                [UpsertAccountOrganization(org.id, uuid4(), Role.superuser.value)]
            )

    def test_when_invalid_org_id__raises_sql_not_found_exception(self):
        account = AccountFactory()
        with pytest.raises(IntegrityError):
            AccountDAO.upsert_account_orgs(
                [UpsertAccountOrganization(uuid4(), account.id, Role.superuser.value)]
            )

    def test_when_invalid_role_name__raises_integrity_error(self):
        org = OrganizationFactory()
        account = AccountFactory()
        with pytest.raises(IntegrityError):
            AccountDAO.upsert_account_orgs(
                [UpsertAccountOrganization(org.id, account.id, "notreal")]
            )

    @staticmethod
    def _assert_upserted_accounts_equal_accounts(upserted_accounts, accounts):
        accounts = [
            UpsertAccountOrganization(u.organization_id, u.account_id, u.role_name)
            for u in accounts
        ]
        assert sorted(upserted_accounts) == sorted(accounts)


class RemoveAccountOrgTest:
    def test_when_exists__deletes_org_account(self):
        org_account = OrganizationAccountFactory()
        AccountDAO.remove_account_org(
            org_account.organization_id, org_account.account_id
        )
        rows = AccountDAO.get_orgs_for_account(org_account.account_id)
        assert rows == []

    def test_when_not_exists__safely_returns(self):
        org = OrganizationFactory()
        account = AccountFactory()
        AccountDAO.remove_account_org(org.id, account.id)
        rows = AccountDAO.get_orgs_for_account(account.id)
        assert rows == []


class GetAccountOrgsTest:
    def test_when_missing_one__raises_sql_not_found_exception(self):
        org_account = OrganizationAccountFactory()
        with pytest.raises(SQLNotFoundException):
            AccountDAO.get_account_orgs(
                [
                    (org_account.organization_id, org_account.account_id),
                    (uuid4(), uuid4()),
                ]
            )

    def test_when_all_found__returns_account_rows(self):
        org_accounts = OrganizationAccountFactory.build_batch(2)
        resp = AccountDAO.get_account_orgs(
            [(ou.organization_id, ou.account_id) for ou in org_accounts]
        )
        assert len(resp) == 2
        assert {row.account_id for row in resp} == {
            ou.account_id for ou in org_accounts
        }

    def test_when_empty__returns_empty(self):
        resp = AccountDAO.get_account_orgs([])
        assert resp == []


class GetUsernameTest:
    def test_when_id_exists__returns_username(self):
        account = AccountFactory.build()
        username = AccountDAO.get_username(account.id)
        assert username.account_id == account.id
        assert username.username == account.email

    def test_when_id_not_exists__raises_not_found_exception(self):
        with pytest.raises(SQLNotFoundException):
            AccountDAO.get_username(uuid4())


class GetUserNamesTest:
    def test_when_name_filter_empty__returns_all_usernames_and_ids(self):
        accounts = AccountFactory.build_batch(3)
        expected_usernames = {Username(a.id, a.email) for a in accounts}
        resp = AccountDAO.get_usernames("", 0, 1000)
        assert expected_usernames.issubset(set(resp))

    @pytest.mark.parametrize(("name_filter",), [("%",), ("e%",), ("%t",), ("e%t",)])
    def test_when_percentage_sign_in_name_filter__finds_all_usernames_with_percentage_sign(
        self, name_filter
    ):
        AccountFactory.build_batch(2)
        target_account = AccountFactory(email="some%thing")
        expected_response = [Username(target_account.id, target_account.email)]

        resp = AccountDAO.get_usernames(name_filter, 0, 1000)
        assert expected_response == resp

    def test_when_name_exact_match__returns_username(self):
        account = AccountFactory()

        resp = AccountDAO.get_usernames(account.email, 0, 1000)
        assert [Username(account.id, account.email)] == resp

    def test_when_no_name_matches__returns_empty(self):
        AccountFactory.build_batch(3)
        name_filter = random_string()
        resp = AccountDAO.get_usernames(name_filter, 0, 1000)
        assert resp == []

    def test_when_multiple_pages__returns_distinct_case_insensitive_sorted_batches(
        self
    ):
        AccountFactory.build_batch(10)

        resp1 = AccountDAO.get_usernames("", 0, 5)
        resp2 = AccountDAO.get_usernames("", 1, 5)

        # ensure distinct batches
        assert set(resp1).isdisjoint(set(resp2))
        usernames1 = [u.username.lower() for u in resp1]
        usernames2 = [u.username.lower() for u in resp2]
        min_in_set2 = min(usernames2)
        max_in_set1 = max(usernames1)

        # Ensure every username in set 2 is greater
        # than every name in set 1 and vice versa
        assert all(max_in_set1 <= username for username in usernames2)
        assert all(min_in_set2 >= username for username in usernames1)


class GetUserNamesByOrgTest:
    def test_when_name_filter_empty__returns_all_usernames_and_ids_for_org(self):
        org = OrganizationFactory()
        org_accounts = OrganizationAccountFactory.build_batch(3, organization=org)

        expected_usernames = {
            Username(oa.account.id, oa.account.email) for oa in org_accounts
        }
        resp = AccountDAO.get_usernames_by_org("", org.id, 0, 1000)
        assert expected_usernames == set(resp)

    @pytest.mark.parametrize(("name_filter",), [("%",), ("e%",), ("%t",), ("e%t",)])
    def test_when_percentage_sign_in_name_filter__finds_all_usernames_with_percentage_sign(
        self, name_filter
    ):
        org = OrganizationFactory()
        OrganizationAccountFactory.build_batch(2, organization=org)
        target = OrganizationAccountFactory(
            organization=org, account__email="some%thing"
        )
        resp = AccountDAO.get_usernames_by_org(name_filter, org.id, 0, 1000)

        expected_response = [Username(target.account_id, target.account.email)]
        assert expected_response == resp

    def test_when_no_name_matches__returns_empty(self):
        org = OrganizationFactory()
        OrganizationAccountFactory.build_batch(2, organization=org)
        name_filter = random_string()

        resp = AccountDAO.get_usernames_by_org(name_filter, org.id, 0, 10)
        assert resp == []

    def test_when_some_matches_in_different_org__only_returns_name_with_org(self):
        org_accounts = OrganizationAccountFactory.build_batch(3)
        org = OrganizationFactory()

        resp = AccountDAO.get_usernames_by_org(
            org_accounts[0].account.email, org.id, 0, 1000
        )
        assert resp == []

    def test_when_second_page__skips_first_results(self):
        org = OrganizationFactory()
        OrganizationAccountFactory.build_batch(10, organization=org)

        resp1 = AccountDAO.get_usernames_by_org("", org.id, 0, 5)
        resp2 = AccountDAO.get_usernames_by_org("", org.id, 1, 5)

        # ensure distinct batches
        assert set(resp1).isdisjoint(set(resp2))
        usernames1 = [u.username.lower() for u in resp1]
        usernames2 = [u.username.lower() for u in resp2]
        min_in_set2 = min(usernames2)
        max_in_set1 = max(usernames1)

        # Ensure every username in set 2 is greater
        # than every name in set 1 and vice versa
        assert all(max_in_set1 <= username for username in usernames2)
        assert all(min_in_set2 >= username for username in usernames1)


class GetAccountInfoTest:
    def test_when_valid__returns_account(self):
        account = AccountFactory()

        resp = AccountDAO.get_account_info(account.id)

        assert resp == Account(username=account.email, id=account.id)

    def test_when_not_found__raises_sqlobject_not_found(self):
        with pytest.raises(SQLNotFoundException):
            AccountDAO.get_account_info(uuid4())



import dataclasses
from enum import Enum
from typing import Dict
from uuid import UUID

from flask import Response
from hou_flask_psycopg2 import Psycopg2Utils

from houflask.auth import AuthorizationService
from houflask.connexion import FlaskConnexionExtension
from houflask.feature_flag.daos.feature_flag_dao import FeatureFlagDAO
from houflask.http import empty_response, json_response, uuid_converter
from ..services.feature_flag_service import FeatureFlagService
import os

__all__ = ["FEATURE_FLAG_VIEW", "FeatureFlagPermission"]
_AUTH = AuthorizationService()


class _FeatureFlagView:
    feature_flag_service: FeatureFlagService

    def init_app(self, db: Psycopg2Utils, connexion_app: FlaskConnexionExtension):
        self.feature_flag_service = FeatureFlagService(FeatureFlagDAO(db))
        _AUTH.init_db(db)
        swagger_file = os.path.join(os.path.dirname(__file__), "..", "swagger-spec.yml")
        connexion_app.add_api(swagger_file, base_path="/v1/flag")


FEATURE_FLAG_VIEW = _FeatureFlagView()


class FeatureFlagPermission(Enum):
    get_feature_flags = "feature-flag:retrieve"
    create_feature_flag = "feature-flag:create"
    delete_feature_flag = "feature-flag:delete"
    update_organization_feature_variant = "feature-flag:organization:set-variant"


@_AUTH.require_global_permissions(FeatureFlagPermission.get_feature_flags)
def get_feature_flags() -> Response:
    feature_flags, feature_variants = (
        FEATURE_FLAG_VIEW.feature_flag_service.get_feature_flags()
    )
    return json_response(
        {
            "feature_flags": (dataclasses.asdict(ff) for ff in feature_flags),
            "variants": (dataclasses.asdict(fv) for fv in feature_variants),
        }
    )


@_AUTH.require_global_permissions(FeatureFlagPermission.create_feature_flag)
def create_feature_flag(body: Dict) -> Response:
    feature_flag = FEATURE_FLAG_VIEW.feature_flag_service.create_feature_flag(
        body["name"], body["default_value"]
    )
    return json_response(dataclasses.asdict(feature_flag), 201)


@_AUTH.require_global_permissions(FeatureFlagPermission.create_feature_flag)
def create_variant(body: Dict) -> Response:
    variant = FEATURE_FLAG_VIEW.feature_flag_service.create_feature_variant(
        body["flag_name"], body["variant"]
    )
    return json_response(dataclasses.asdict(variant), 201)


@_AUTH.require_global_permissions(FeatureFlagPermission.delete_feature_flag)
def delete_variant(variant_id: UUID) -> Response:
    FEATURE_FLAG_VIEW.feature_flag_service.delete_feature_variant(variant_id)
    return empty_response()


@_AUTH.require_global_permissions(
    FeatureFlagPermission.update_organization_feature_variant
)
def set_organization_variant(organization_id: UUID, body: Dict) -> Response:
    feature_variant_organization = FEATURE_FLAG_VIEW.feature_flag_service.update_organization_variant(
        organization_id, body["variant_id"], body["flag_name"]
    )
    return json_response(dataclasses.asdict(feature_variant_organization))



from typing import List, Tuple, Union
from uuid import UUID

from ..daos.feature_flag_dao import (
    FeatureFlag,
    FeatureFlagDAO,
    FeatureVariant,
    FeatureVariantOrganization,
    NewFeatureVariant,
)


class FeatureFlagService:
    def __init__(self, feature_flag_dao: FeatureFlagDAO):
        self.dao = feature_flag_dao

    def get_feature_flags(self) -> Tuple[List[FeatureFlag], List[FeatureVariant]]:
        feature_flags = self.dao.get_feature_flags()
        feature_variants = self.dao.get_feature_variants()
        return feature_flags, feature_variants

    def create_feature_flag(self, flag_name: str, default_value) -> FeatureFlag:
        return self.dao.create_feature_flag(flag_name, default_value)

    def create_feature_variant(self, flag_name: str, variant) -> FeatureVariant:
        return self.dao.create_feature_variants(
            [NewFeatureVariant(flag_name=flag_name, variant=variant)]
        )[0]

    def delete_feature_variant(self, variant_id: UUID):
        self.dao.delete_feature_variants([variant_id])

    def update_organization_variant(
        self, organization_id: UUID, variant_id: Union[UUID, None], flag_name: str
    ) -> FeatureVariantOrganization:
        self.dao.delete_feature_variant_organization(organization_id, flag_name)
        if variant_id is not None:
            return self.dao.create_feature_variant_organization(
                flag_name, variant_id, [organization_id]
            )[0]
        return FeatureVariantOrganization(
            organization_id=organization_id, variant_id=variant_id, flag_name=flag_name
        )



import logging

from hou_flask_psycopg2 import Psycopg2Utils
from status_checker import StatusChecker
import os

from houflask.connexion import FlaskConnexionExtension
from houflask.http import json_response

LOG = logging.getLogger(__name__)


class _MetaView:
    db: Psycopg2Utils
    status_checker: StatusChecker
    app_version: str

    def init_app(
        self,
        db: Psycopg2Utils,
        connexion_app: FlaskConnexionExtension,
        app_version: str,
        **additional_checks
    ):
        self.db = db
        self.status_checker = StatusChecker(
            database=self._check_database, **additional_checks
        )
        self.app_version = app_version
        swagger_file = os.path.join(os.path.dirname(__file__), "..", "swagger-spec.yml")
        connexion_app.add_api(swagger_file, base_path="/v1/meta")

    # pylint: disable=broad-except
    def _check_database(self):
        """
        Checks if the database is available

        :return: Return a dictionary with at least
            a key 'available' corresponding to a boolean
        :rtype: dict{str:object}
        """
        try:
            with self.db.cursor() as cur:
                cur.execute("SELECT 1")
                cur.fetchall()
        except Exception as exc:
            return {"available": False, "message": str(exc)}
        else:
            return {"available": True, "message": "ready"}


META_VIEW = _MetaView()


def status():
    """
    Gets the status of this service and its dependencies
    """
    status_dict = META_VIEW.status_checker.status()
    status_code = 200 if status_dict["failure_count"] == 0 else 500
    return json_response(status_dict, status_code)


def version():
    """
    Gets the version of this service
    """
    version_info = {"version": META_VIEW.app_version}
    return json_response(version_info)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from setuptools import setup, find_packages

with open("README.md") as readme_file:
    readme = readme_file.read()

with open("HISTORY.md") as history_file:
    history = history_file.read()


setup(
    name="hou-flask",
    version="0.0.9",
    description="Basic authentication and authorization application",
    long_description=readme + "\n\n" + history,
    author="Tim Martin",
    author_email="oss@timmartin.me",
    url="https://github.com/timmartin19/hou-flask",
    packages=find_packages(),
    package_data={"hou-flask": ["README.md", "HISTORY.md"]},
    include_package_data=True,
    install_requires=[
        "hou-flask-psycopg2",
        "python-rapidjson",
        "flask",
        "connexion",
        "werkzeug",
        "ultra-config",
        "jsonschema<3.0.0",
        "beaker",
        "boto3",
        "python-jose",
        "flask-wtf",
        "pystatuschecker",
        "factory-boy",
    ],
    zip_safe=False,
    keywords="houflask",
    classifiers=[
        "Development Status :: 2 - Pre-Alpha",
        "Intended Audience :: Developers",
        "Natural Language :: English",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
    ],
    test_suite="tests",
)



class GetUsernameTest:
    def test_when_id_exists__returns_username(self):
        account = AccountFactory.build()
        username = AccountDAO.get_username(account.id)
        assert username.account_id == account.id
        assert username.username == account.email

    def test_when_id_not_exists__raises_not_found_exception(self):
        with pytest.raises(SQLNotFoundException):
            AccountDAO.get_username(uuid4())


class GetUserNamesTest:
    def test_when_name_filter_empty__returns_all_usernames_and_ids(self):
        accounts = AccountFactory.build_batch(3)
        expected_usernames = {Username(a.id, a.email) for a in accounts}
        resp = AccountDAO.get_usernames("", 0, 1000)
        assert expected_usernames.issubset(set(resp))

    @pytest.mark.parametrize(("name_filter",), [("%",), ("e%",), ("%t",), ("e%t",)])
    def test_when_percentage_sign_in_name_filter__finds_all_usernames_with_percentage_sign(
        self, name_filter
    ):
        AccountFactory.build_batch(2)
        target_account = AccountFactory(email="some%thing")
        expected_response = [Username(target_account.id, target_account.email)]

        resp = AccountDAO.get_usernames(name_filter, 0, 1000)
        assert expected_response == resp

    def test_when_name_exact_match__returns_username(self):
        account = AccountFactory()

        resp = AccountDAO.get_usernames(account.email, 0, 1000)
        assert [Username(account.id, account.email)] == resp

    def test_when_no_name_matches__returns_empty(self):
        AccountFactory.build_batch(3)
        name_filter = random_string()
        resp = AccountDAO.get_usernames(name_filter, 0, 1000)
        assert resp == []

    def test_when_multiple_pages__returns_distinct_case_insensitive_sorted_batches(
        self
    ):
        AccountFactory.build_batch(10)

        resp1 = AccountDAO.get_usernames("", 0, 5)
        resp2 = AccountDAO.get_usernames("", 1, 5)

        # ensure distinct batches
        assert set(resp1).isdisjoint(set(resp2))
        usernames1 = [u.username.lower() for u in resp1]
        usernames2 = [u.username.lower() for u in resp2]
        min_in_set2 = min(usernames2)
        max_in_set1 = max(usernames1)

        # Ensure every username in set 2 is greater
        # than every name in set 1 and vice versa
        assert all(max_in_set1 <= username for username in usernames2)
        assert all(min_in_set2 >= username for username in usernames1)


class GetAccountInfoTest:
    def test_when_valid__returns_account(self):
        account = AccountFactory()

        resp = AccountDAO.get_account_info(account.id)

        assert resp == Account(username=account.email, id=account.id)

    def test_when_not_found__raises_sqlobject_not_found(self):
        with pytest.raises(SQLNotFoundException):
            AccountDAO.get_account_info(uuid4())
